<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2020.1/2030377
 */
namespace Comments\Model;

use Application\Connection\ConnectionFactory;
use Application\Model\AbstractDAO;
use Application\Model\IModelDAO;
use Application\Permissions\IpProtects;
use P4\Connection\ConnectionInterface;
use P4\Model\Fielded\Iterator;

/**
 * Class CommentDAO to handle access to Perforce comment data
 * @package Changes\Model
 */
class CommentDAO extends AbstractDAO
{
    // The Perforce class that handles comments
    const MODEL        = Comment::class;
    const REVIEW_TOPIC = 'reviews';
    const CHANGE_TOPIC = 'changes';

    /**
     * As the fetchAll function in the model takes extra param of protections we have to write
     * this wrapper to handle that.
     * Retrieves all records that match the passed options.
     * Extends parent to compose a search query when fetching by topic.
     *
     * @param array           $options      an optional array of search conditions and/or options
     *                                      supported options are:
     *                                      FETCH_MAXIMUM - set to integer value to limit to the first
     *                                      max' number of entries.
     *                                      FETCH_AFTER - set to an id _after_ which we start collecting
     *                                      FETCH_BY_TOPIC - set to a 'topic' id to limit results
     *                                      FETCH_BY_IDS - provide an array of ids to fetch.
     *                                      not compatible with FETCH_SEARCH or FETCH_AFTER.
     *                                      FETCH_BY_READ_BY - includes comments that have been read by the users
     *                                      FETCH_BY_UNREAD_BY - includes comments that have not been read by the
     *                                      users
     * @param ConnectionInterface|null $connection the connection to set on the filer
     * @param null            $protects            the users protections list.
     * @return  Iterator                           the list of zero or more matching comment objects
     */
    public function fetchAllWithProtection(
        array $options = [],
        ConnectionInterface $connection = null,
        $protects = null
    ) {
        // By default this simply passes on to model, with extra option of protections.
        return call_user_func(
            static::MODEL  . '::fetchAll',
            $options,
            $this->getConnection($connection),
            $protects
        );
    }

    /**
     * Fetch all the comments for a given Review Id.
     * @param String                   $reviewId    the review ID
     * @return mixed
     */
    public function fetchAllReviewComments($reviewId)
    {
        $p4Admin    = $this->services->get(ConnectionFactory::P4_ADMIN);
        $topic      = self::REVIEW_TOPIC.'/'.$reviewId;
        $reviewDAO  = $this->services->get(IModelDAO::REVIEW_DAO);
        $ipProtects = $this->services->get(IpProtects::IP_PROTECTS);
        // To check access.
        $reviewDAO->fetch($reviewId, $p4Admin);

        $options  = [Comment::FETCH_BY_TOPIC => [$topic]];
        $comments = $this->fetchAllWithProtection($options, $p4Admin, $ipProtects);
        return $comments->toArray();
    }
}
