<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2020.1/2030377
 */
namespace TestIntegration\Listener;

use Activity\Model\Activity;
use Application\Config\IDao;
use Application\Connection\ConnectionFactory;
use Application\Log\SwarmLogger;
use Application\Model\IModelDAO;
use Events\Listener\AbstractEventListener;
use Events\Listener\ListenerFactory;
use Reviews\Model\Review;
use TestIntegration\Model\ITestRun;
use TestIntegration\Model\TestRun as Model;
use Laminas\EventManager\Event;
use Record\Exception\NotFoundException;
use Exception;

/**
 * Class TestRun, listens to TestRun events raised based on queue tasks
 * @package TestIntegration\Listener
 */
class TestRun extends AbstractEventListener
{
    const UPGRADED_ACTION = 'upgraded';
    /**
     * Listener for TestRun add/update. Updates the overall test status on a review if the
     * TestRun changed is for the latest version
     * @param Event $event  Zend event
     * @return bool true if a review was updated, false otherwise
     */
    public function update(Event $event)
    {
        $updated = true;
        $data    = $event->getParam('data');
        $id      = $data[Model::FIELD_CHANGE];
        $version = $data[Model::FIELD_VERSION];
        try {
            $p4Admin = $this->services->get(ConnectionFactory::P4_ADMIN);
            $review  = Review::fetch($id, $p4Admin);
            if ($version == $review->getHeadVersion()) {
                $dao    = $this->services->get(IDao::TEST_RUN_DAO);
                $status = $dao->calculateTestStatus($review->getTestRuns(), $p4Admin);
                if ($status) {
                    $review->setTestStatus($status)->save();
                }
            }
        } catch (NotFoundException $e) {
            $updated = false;
            $logger  = $this->services->get(SwarmLogger::SERVICE);
            $logger->err(sprintf("[%s]: Review [%s] not found on TestRun update", get_class($this), $id));
        }
        return $updated;
    }

    /**
     * Function to upgrade existing TestRun instances triggered by an upgrade task in the queue
     * @param Event $event  Laminas event
     */
    public function upgradeTestRuns(Event $event)
    {
        $logPrefix = self::class . ':' . __FUNCTION__ . ':';
        $services  = $this->services;
        $logger    = $this->logger;
        try {
            $p4Admin    = $services->get(ConnectionFactory::P4_ADMIN);
            $testRunDao = $services->get(IModelDAO::TEST_RUN_DAO);
            $data       = $event->getParam(ListenerFactory::DATA);
            $upgraded   = [];
            foreach (array_filter(
                iterator_to_array($testRunDao->fetchAll([], $p4Admin)),
                function ($testRun) use ($data) {
                    return $data[ITestRun::FIELD_UPGRADE] > (int)$testRun->get(ITestRun::FIELD_UPGRADE);
                }
            ) as $needsUpgrade) {
                $description = $needsUpgrade->getTest() . '(' . $needsUpgrade->getId() . ')';
                try {
                    $logger->trace(sprintf("%s Upgrading [%s]", $logPrefix, $description));
                    $testRunDao->save($needsUpgrade);
                    $upgraded[] = $description;
                } catch (Exception $e) {
                    $logger->err(sprintf("%s Failed to upgrade test run [%s]", $logPrefix, $description));
                    $logger->err($e->getMessage());
                }
            }
            if (count($upgraded) > 0) {
                // Only log and create activity if there was work to be done
                $upgradeTargets = count($upgraded) . ' test run(s) [' . implode(', ', $upgraded) . ']';
                $event->setParam(
                    'activity',
                    (new Activity())->set(
                        [
                            'action' => self::UPGRADED_ACTION,
                            'type' => 'testRun',
                            'user' => $p4Admin->getUser(),
                            'target' => $upgradeTargets
                        ]
                    )
                );
                $logger->info(sprintf("%s Upgraded %s", $logPrefix, $upgradeTargets));
            }
        } catch (Exception $e) {
            $logger->err($e->getMessage());
        }
    }
}
