<?php
/**
 * Perforce Swarm
 *
 * @copyright   2013-2020 Perforce Software. All rights reserved.
 * @license     Please see LICENSE.txt in top-level readme folder of this distribution.
 * @version     2020.1/2030377
 */

namespace TestIntegration\Service;

use Application\Factory\InvokableService;
use P4\Model\Fielded\Iterator;
use Projects\Model\Project;
use Record\Key\AbstractKey;
use TestIntegration\Model\ITestDefinition;
use TestIntegration\Model\ITestRun as ITestRunModel;

/**
 * Interface ITestExecutor
 * @package TestIntegration\Service
 */
interface ITestExecutor extends InvokableService
{
    const NAME = 'testExecutorService';
    // Test run name for project tests is 'project:<project id>:<test name>
    const PROJECT_TEST_SEPARATOR = ':';
    const PROJECT_PREFIX         = 'project' . self::PROJECT_TEST_SEPARATOR;
    const PROJECT_TEST_FORMAT    = self::PROJECT_PREFIX . "%s" . self::PROJECT_TEST_SEPARATOR . "%s";

    /**
     * Runs all tests associated with a given review, its projects and their workflows
     * @param AbstractKey $review           review
     * @param array       $affectedProjects maps each affected project's id to its branch ids
     *                                      Ex. ['proj1' => ['branchA', 'branchB'], 'proj2' => ['branchY', 'branchX']]
     * @param Iterator    $projects         instances of Project models, directly associated with the review
     */
    public function runReviewTests(AbstractKey $review, array $affectedProjects, Iterator $projects);

    /**
     * Runs all the tests associated with the global workflow and any other workflows linked to affected projects
     * @param array $testsForWorkflows      keys of event names with values of test ids. For example:
     * [
     *     'onUpdate' => [4, 7, 9],
     *     'onSubmit' => [1, 4, 10]
     * ]
     * @param array $previouslyPassedNames  any test names that passed on a previous revision of the review. When there
     *                                      is no content change previously passed tests will not be executed.
     * @return mixed
     */
    public function runWorkflowTests($testsForWorkflows, &$previouslyPassedNames = []);

    /**
     * Runs the project tests associated with a review
     * @param array $previouslyPassedNames  any test names that passed on a previous revision of the review. When there
     *                                      is no content change previously passed tests will not be executed.
     */
    public function runProjectTests($previouslyPassedNames = []);

    /**
     * Runs a single test
     * @param ITestDefinition $testDefinition   test definition
     * @param array           $fields           key-value pairs, used to build the query string for the CI request
     * @param ITestRunModel   $testRun          test run record
     * @return array|null request values
     */
    public function runTest(ITestDefinition $testDefinition, array $fields, ITestRunModel $testRun);

    /**
     * Runs a single project test
     * @param Project         $project          project object with test definition information
     * @param array           $fields           key-value pairs, used to build the query string for the CI request
     * @param ITestRunModel   $testRun          test run record
     * @param array           $testStartTimes   holds the start times of all the project tests
     * @return array|null request values
     */
    public function runProjectTest(Project $project, array $fields, ITestRunModel $testRun, array &$testStartTimes);
}
